/*
 *      Program do konwersji logow Gadu do ekg.
 *      Implementacja klasy Convert - gwnej klasy programu.
 *
 *      copyright (c) 2004 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "convert.h"
#include "stuff.h"

#include <ctime>

const string & Convert::prepareTemporaryContacts(const string & original)
{
	unsigned int i = 0, l;
	string line;
	static string tmpContacts;

	while (1) {
		l = original.find("\n", i);
		if (l == string::npos)
			break;

		// wycinamy kontakt
		l -= i;
		line = original.substr(i, l);
		i += l + 1;

		Contact c(line);
		c.show = "__gg-convert__" + c.show;
		tmpContacts += c;
		tmpContacts += '\n';
	}

	return tmpContacts;
}


const string & Convert::prepareTemporaryContactsFromFile(const string & fname)
{
	unsigned int length;
	char * buffer;
	    
	ifstream input(fname.c_str());

	input.seekg(0, ios::end);
	length = input.tellg();
	input.seekg(0, ios::beg);
			
	buffer = new char [length];
			    
	input.read(buffer,length);
	input.close();

	const string & str = prepareTemporaryContacts(buffer);
	delete [] buffer;

	return str;
}


void Convert::convertLog(const string & fname, ostream & out)
{
	_currentLineGood = false;
	ifstream currentFile(fname.c_str());

	while (readChat(currentFile, out))
		;
}


void Convert::convertLog(istream & in, ostream & out)
{
	_currentLineGood = false;

	while (readChat(in, out))
		;
}



void Convert::readContacts(const string & fname)
{
	ifstream in(fname.c_str());
	string line;
	while (1) {
		getline(in, line);
		if (!in)
			break;

		Contact c(line);
		_contacts[c.show] = c.uin;
	}
}


bool Convert::readChat(istream & in, ostream & out)
{
	getLine(in);
	while (_currentLine.compare(0, 14, "*- Rozmowa z: ") &&
		in)
	{
		getNewLine(in);
	}

	// jesli koniec pliku
	if (!in)
		return false;

	// "*- Rozmowa z: __gg-conver__"
	//  012345678901234567890123456
	_partner.nick = _currentLine.substr(28,_currentLine.size()-27-5);
	_partner.uin  = getUin(_partner.nick);

	getNewLine(in);
	getNewLine(in);
	while (readMessage(in, out))
		;

	return true;
}


bool Convert::readMessage(istream & in, ostream & out)
{
	getLine(in);
	convertHeader(out);
	string msg;
	unsigned int lineNo = 0;

	// teraz tresc wiadomosci
	getNewLine(in);
	while (_currentLine.compare(0, 14, "*- Rozmowa z: ") &&
	       _currentLine.compare(0, 14, "__gg-convert__") &&
	       in)
	{
		if (lineNo++)
			msg += "\n";
		msg += _currentLine;
		getNewLine(in);
	}

	// trzeba usunas ostatni \n - nie nalezy on do wiadomosci
	if (msg.size() && !_currentLine.compare(0, 14, "*- Rozmowa z: "))
		msg.erase(msg.size()-2);

	out << '\"';
	for (string::const_iterator i=msg.begin(); i!=msg.end(); ++i) {
		switch (*i) {
			case 0x0D: /* ^M == \r */
				break;
			case '\n':
				out << "\\r\\n";
				break;
			case '\"':
			case '\'':
			case '\\':
				out << '\\';
			default:
				out << *i;
		}
	}
	out << "\"\n";

	return (_currentLine.compare(0, 14, "__gg-convert__") == 0);
}


void Convert::convertHeader(ostream & out)
{
	#define line _currentLine
	#define me _ownerNick

	string nick = line .substr(14,line.find(" ")-14);
	string time = line .substr(line.find("(")+1, string::npos);

	tm t;
	t.tm_mday = (time[0]-'0')*10 + time[1]-'0';
	t.tm_mon  = (time[3]-'0')*10 + time[4]-'0' - 1;
	t.tm_year = (time[6]-'0')*1000 + (time[7]-'0')*100 + (time[8]-'0')*10 + time[9]-'0' - 1900;
	t.tm_hour = (time[11]-'0')*10 + time[12]-'0';
	t.tm_min  = (time[14]-'0')*10 + time[15]-'0';
	t.tm_sec  = (time[17]-'0')*10 + time[18]-'0';

	bool sent = (nick == me);

	string nline = sent ? "chatsend," : "chatrecv,";
	nline += _partner.uin;	nline += ",";
	nline += _partner.nick;	nline += ",";
	nline += text_cast(mktime(&t));
	nline += ",";

	if (!sent) { nline += text_cast(mktime(&t)); nline += ","; }

	out << nline;

	#undef line
	#undef me
}


void Convert::getLine(istream & in)
{
	if (!_currentLineGood) {
		getline(in, _currentLine);
		_currentLineGood = true;
	}
}


void Convert::getNewLine(istream & in)
{
	getline(in, _currentLine);
	_currentLineGood = true;
}


const string & Convert::getUin(const string & nick)
{
	static string uin = "0";

	map<string, string>::iterator i = _contacts.find(nick);
	if (i != _contacts.end())
		return i->second;

	return uin;
}
