/*
 *      Program do konwersji logow Gadu do ekg.
 *      Gwna klasa programu.
 *
 *      copyright (c) 2004 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef __CONVERT_H__
#define __CONVERT_H__

#include <string>
#include <fstream>
#include <iostream>
#include <map>
using namespace std;

class Convert {
public:

	Convert(const string & nick) : _ownerNick(nick),_currentLineGood(false) {}

	/** 
	 * Zamiana kontaktow Gadu na tymczasowe sluzace
	 * poprawnej interpretacji wyeksportowanych z Gadu logow.
	 *
	 * @param original lista kontaktow gadu
	 * @return tymczasowa lista kontaktow
	 */
	const string & prepareTemporaryContacts(const string & original);
	/**
	 * Otwiera podany plik z kontaktami Gadu i generuje liste tymczasowa.
	 *
	 * @param fname nazwa pliku z kontaktami
	 * @return tymczasowa lista kontaktow
	 */
	const string & prepareTemporaryContactsFromFile(const string & fname);
	/**
	 * Przetwarza plik z rozmowami i konwertuje go do formatu ekg.
	 *
	 * @param fname nazwa plik z rozmowami
	 * @param out strumien wyjsciowy
	 */
	void convertLog(const string & fname, ostream & out);
	/**
	 * Przetwarza plik z rozmowami i konwertuje go do formatu ekg.
	 *
	 * @param in strumien wejsciowy
	 * @param out strumien wyjsciowy
	 */
	void convertLog(istream & in, ostream & out);

	/**
	 * Wczytuje liste kontaktow.
	 * @param fname nazwa pliku z kontaktami
	 */
	void readContacts(const string & fname);

private:

	// Klasa kontaktu - do uzytku wewnetrznego przy analizowaniu listy kontaktow
	struct Contact {
		Contact(const string & line);
		operator string ();
		// dane kontaktu
		string name,surname,nick,show,phone,uin,email,rubbish1,rubbish2;
	};


	// Wczytuje pojedyncza rozmowe. Zwraca true jesli mozna dalej czytac i false,
	// jesli to juz koniec pliku.
	bool readChat(istream & in, ostream & out);
	// Wczytuje wiadomosc. Zwraca true jesli mozna czytac nastepna wiadomosc,
	// false jesli koniec pliku lub nastepny czat.
	bool readMessage(istream & in, ostream & out);
	// Wczytuje jedna linie z biezacego pliku - zakladamy poprawnosc pliku.
	void getLine(istream & in);
	// Wymusza wczytanie nowej linii.
	void getNewLine(istream & in);
	// Konwertuje naglowek wpisu i wypisuje go do strumienia.
	void convertHeader(ostream & out);
	// Zwraca UIN dla podanego nick'a.
	const string & getUin(const string & nick);


	// Aktualnie przetwarzana linia - wspoldzielona przez metody.
	string _currentLine;
	// Nick wlasciciela konta gadu - czyli nazwa profilu Gadu.
	string  _ownerNick;
	// Nick partnera z aktualnie przetwarzanej rozmowy.
	struct { string nick, uin; } _partner;
	// Czy biezaca linia jest aktualna - jesli nie, trzeba pobrac nowa.
	bool _currentLineGood;
	// Mapa z kontaktami -> map[nick] = uin
	map<string,string> _contacts;
};
#endif /* __CONVERT_H__ */
